from smc import getMinimalArgParser, getRobotFromArgs
from smc.util.define_random_goal import getRandomlyGeneratedGoal
from smc.control.cartesian_space import getClikArgs
from smc.robots.utils import defineGoalPointCLI
from smc.control.cartesian_space.cartesian_space_point_to_point import moveLDualArm

import numpy as np
import argparse
import pinocchio as pin


def get_args():
    parser = getMinimalArgParser()
    parser.description = "Run closed loop inverse kinematics \
    of various kinds. Make sure you know what the goal is before you run!"
    parser = getClikArgs(parser)
    parser.add_argument(
        "--randomly-generate-goal",
        action=argparse.BooleanOptionalAction,
        default=False,
        help="if true, rand generate a goal, if false you type it in via text input",
    )
    args = parser.parse_args()
    return args


if __name__ == "__main__":
    args = get_args()
    robot = getRobotFromArgs(args)
    if args.randomly_generate_goal:
        T_w_goal = getRandomlyGeneratedGoal(args)
        if args.visualizer:
            robot.visualizer_manager.sendCommand({"Mgoal": T_w_goal})
    else:
        T_w_goal = defineGoalPointCLI(robot)
        T_w_goal.rotation = np.eye(3)
    T_absgoal_lgoal = pin.SE3.Identity()
    T_absgoal_lgoal.translation[1] = 0.1
    T_absgoal_rgoal = pin.SE3.Identity()
    T_absgoal_rgoal.translation[1] = -0.1
    moveLDualArm(args, robot, T_w_goal, T_absgoal_lgoal, T_absgoal_rgoal)

    if args.real:
        robot.stopRobot()

    if args.visualizer:
        robot.killManipulatorVisualizer()

    if args.save_log:
        robot._log_manager.saveLog()
