# PYTHON_ARGCOMPLETE_OK
from smc.control.optimal_control.util import get_OCP_args
from smc import getMinimalArgParser, getRobotFromArgs
from smc.control.optimal_control.point_to_point_croco_ocp import (
    SingleArmIKOCP,
)
from smc.control.joint_space import followKinematicJointTrajP

import pinocchio as pin
import numpy as np
import argcomplete


def get_args():
    parser = getMinimalArgParser()
    parser = get_OCP_args(parser)
    argcomplete.autocomplete(parser)
    args = parser.parse_args()
    return args


if __name__ == "__main__":
    args = get_args()
    robot = getRobotFromArgs(args)
    # TODO: put this back for nicer demos
    # Mgoal = robot.defineGoalPointCLI()
    T_w_goal = pin.SE3.Random()

    if args.visualizer:
        # TODO document this somewhere
        robot.visualizer_manager.sendCommand({"Mgoal": T_w_goal})

    # create and solve the optimal control problem of
    # getting from current to goal end-effector position.
    # reference is position and velocity reference (as a dictionary),
    # while solver is a crocoddyl object containing a lot more information
    # starting state
    x0 = np.concatenate([robot.q, robot.v])
    # this shouldn't really depend on x0 but i can't be bothered
    ocp = SingleArmIKOCP(args, robot, x0, T_w_goal)
    ocp.solveInitialOCP(x0)
    reference = ocp.getSolvedReference()

    # NOTE: IF YOU PLOT SOMETHING OTHER THAN REAL-TIME PLOTTING FIRST IT BREAKS EVERYTHING
    #    if args.solver == "boxfddp":
    #        log = solver.getCallbacks()[1]
    #        crocoddyl.plotOCSolution(log.xs, log.us, figIndex=1, show=True)

    # we need a way to follow the reference trajectory,
    # both because there can be disturbances,
    # and because it is sampled at a much lower frequency
    followKinematicJointTrajP(args, robot, reference)

    print("final position:")
    print(robot.T_w_e)

    if args.real:
        robot.stopRobot()

    if args.visualizer:
        robot.killManipulatorVisualizer()

    if args.save_log:
        robot._log_manager.saveLog()
        robot._log_manager.plotAllControlLoops()
