from smc import getMinimalArgParser, getRobotFromArgs
from smc.control.optimal_control.croco_mpc_point_to_point import (
    CrocoDualEEP2PMPC,
)
from smc.control.optimal_control.util import get_OCP_args
from smc.control.cartesian_space import getClikArgs
from smc.util.define_random_goal import getRandomlyGeneratedGoal
import pinocchio as pin
import argcomplete


def get_args():
    parser = getMinimalArgParser()
    parser = get_OCP_args(parser)
    parser = getClikArgs(parser)  # literally just for goal error
    argcomplete.autocomplete(parser)
    args = parser.parse_args()
    return args


if __name__ == "__main__":
    args = get_args()
    robot = getRobotFromArgs(args)
    T_w_absgoal = getRandomlyGeneratedGoal(args)
    T_absgoal_lgoal = pin.SE3.Identity()
    T_absgoal_lgoal.translation[1] = 0.1
    T_absgoal_rgoal = pin.SE3.Identity()
    T_absgoal_rgoal.translation[1] = -0.1

    if args.visualizer:
        # TODO: document this somewhere
        robot.visualizer_manager.sendCommand({"Mgoal": T_w_absgoal})

    CrocoDualEEP2PMPC(args, robot, T_w_absgoal, T_absgoal_lgoal, T_absgoal_rgoal)

    if args.real:
        robot.stopRobot()

    if args.visualizer:
        robot.killManipulatorVisualizer()

    if args.save_log:
        robot._log_manager.saveLog()
        robot._log_manager.plotAllControlLoops()
